// -*- C++ -*-

/* 
 * GChemPaint
 * bond.cc 
 *
 * Copyright (C) 2001-2003
 *
 * Developed by Jean Brfort <jean.brefort@ac-dijon.fr>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "config.h"
#include "globals.h"
#include "bond.h"
#include "atom.h"
#include "cycle.h"
#include "settings.h"
#include "document.h"
#include <math.h>

gcpBond::gcpBond(): Bond()
{
	m_CoordsCalc = false;
	m_type = NormalBondType;
}

gcpBond::gcpBond(gcpAtom* first, gcpAtom* last, unsigned char order): Bond(first, last, order)
{
	m_CoordsCalc = false;
	m_type = NormalBondType;
}

gcpBond::~gcpBond()
{
}

/*gcpAtom* gcpBond::GetAtom(int which)
{
	switch (which)
	{
	case 0: return m_Begin;
	case 1: return m_End;
	default: return NULL;
	}
}

gcpAtom* gcpBond::GetAtom(gcpAtom* pAtom, int which)
{
	if (pAtom == m_Begin) return m_End;
	if (pAtom == m_End) return m_Begin;
	return NULL;
}

unsigned char gcpBond::GetOrder()
{
	return m_order;
}

void gcpBond::SetOrder(unsigned char Order)
{
	m_order = Order;
}*/

void gcpBond::SetType(gcpBondType type)
{
	m_type = type;
	m_CoordsCalc = false;
	if (m_type != NormalBondType) m_order = 1;
}

double gcpBond::GetAngle2D(gcpAtom* pAtom)
{
	double x1, y1, x2, y2;
	m_Begin->GetCoords(&x1, &y1);
	m_End->GetCoords(&x2, &y2);
	x2 -= x1;
	y2 -= y1;
	double length = square(x2) + square(y2);
	if (length == 0.0) return HUGE_VAL;
	if (pAtom == m_Begin) return atan2(- y2, x2) * 90 / 1.570796326794897;
	else if (pAtom == m_End) return atan2(y2, - x2) * 90 / 1.570796326794897;
	return HUGE_VAL;
}

double gcpBond::GetAngle2DRad(gcpAtom* pAtom)
{
	double x1, y1, x2, y2;
	m_Begin->GetCoords(&x1, &y1);
	m_End->GetCoords(&x2, &y2);
	x2 -= x1;
	y2 -= y1;
	double length = square(x2) + square(y2);
	if (length == 0.0) return HUGE_VAL;
	if (pAtom == m_Begin) return atan2(- y2, x2);
	else if (pAtom == m_End) return atan2(y2, - x2);
	return HUGE_VAL;
}

gcpCycle* gcpBond::GetFirstCycle(std::list<gcpCycle*>::iterator& i, gcpCycle * pCycle)
{
	i = m_Cycles.begin();
	return GetNextCycle(i, pCycle);
}

gcpCycle* gcpBond::GetNextCycle(std::list<gcpCycle*>::iterator& i, gcpCycle * pCycle)
{
	if (*i == pCycle) i++;
	if (i == m_Cycles.end()) return NULL;
	pCycle = *i;
	i++;
	return pCycle;
}

bool gcpBond::IsInCycle(gcpCycle* pCycle)
{
	std::list<gcpCycle*>::iterator i;
	for (i = m_Cycles.begin(); i != m_Cycles.end(); i++)
		if ((*i) == pCycle) return true;
	return false;
}

bool gcpBond::GetLine2DCoords(unsigned Num, double* x1, double* y1, double* x2, double* y2)
{
	if ((Num == 0) || (Num > m_order)) return false;
	if (!m_CoordsCalc)
	{
		m_Begin->GetCoords(x1, y1);
		m_End->GetCoords(x2, y2);
		double dx = *x2 - *x1, dy = *y2 - *y1;
		double l = sqrt(square(dx) + square(dy));
		dx *= (DefaultBondDist / l);
		dy *= (DefaultBondDist / l);
		if (m_order & 1)
		{
			m_coords[0] = *x1;
			m_coords[1] = *y1;
			m_coords[2] = *x2;
			m_coords[3] = *y2;
			if (m_order == 3)
			{
				m_coords[4] = *x1 - dy;
				m_coords[5] = *y1 + dx;
				m_coords[6] = *x2 - dy;
				m_coords[7] = *y2 + dx;
				m_coords[8] = *x1 + dy;
				m_coords[9] = *y1 - dx;
				m_coords[10] = *x2 + dy;
				m_coords[11] = *y2 - dx;
			}
		}
		else if ((m_order == 2) && IsCyclic())
		{
			m_coords[0] = *x1;
			m_coords[1] = *y1;
			m_coords[2] = *x2;
			m_coords[3] = *y2;
			gcpCycle* pCycle;
			if (IsCyclic() > 1)
			{
				//Search prefered cycle
				std::list<gcpCycle*>::iterator i = m_Cycles.begin();
				pCycle = *i;
				for (; i != m_Cycles.end(); i++)
					if (pCycle->IsBetterForBonds(*i)) pCycle = *i;
			}
			else pCycle = m_Cycles.front();
			double a0 = atan2(*y1 - *y2, *x2 - *x1), a1, a2;
			pCycle->GetAngles2D(this, &a1, &a2);
			if (sin(a0 - a1) * sin (a0 - a2) > 0)
			{
				double sign = sin(a0 - a1) > 0.0 ? 1.0 : -1.0;
				double tanb = fabs(tan((3.141592653589793238462643 - a0 + a1) / 2)), cosa = cos(a0), sina = sin(a0);
				m_coords[4] = *x1 + DefaultBondDist * cosa * tanb - dy * sign;
				m_coords[5] = *y1 + dx * sign - DefaultBondDist * sina * tanb;
				tanb = fabs(tan((a2 - a0) / 2));
				m_coords[6] = *x2 - DefaultBondDist * cosa * tanb - dy * sign;
				m_coords[7] = *y2 + dx * sign + DefaultBondDist * sina * tanb;
			}
			else
			{
				m_coords[0] = *x1 - dy / 2;
				m_coords[1] = *y1 + dx / 2;
				m_coords[2] = *x2 - dy / 2;
				m_coords[3] = *y2 + dx / 2;
				m_coords[4] = *x1 + dy / 2;
				m_coords[5] = *y1 - dx / 2;
				m_coords[6] = *x2 + dy / 2;
				m_coords[7] = *y2 - dx / 2;
			}
		}
		else
		{
			m_coords[0] = *x1 - dy / 2;
			m_coords[1] = *y1 + dx / 2;
			m_coords[2] = *x2 - dy / 2;
			m_coords[3] = *y2 + dx / 2;
			m_coords[4] = *x1 + dy / 2;
			m_coords[5] = *y1 - dx / 2;
			m_coords[6] = *x2 + dy / 2;
			m_coords[7] = *y2 - dx / 2;
			if (m_order == 4)
			{
				m_coords[8] = *x1 - dy * 1.5;
				m_coords[9] = *y1 + dx * 1.5;
				m_coords[10] = *x2 - dy * 1.5;
				m_coords[11] = *y2 + dx * 1.5;
				m_coords[12] = *x1 + dy * 1.5;
				m_coords[13] = *y1 - dx * 1.5;
				m_coords[14] = *x2 + dy * 1.5;
				m_coords[15] = *y2 - dx * 1.5;
			}
		}
		m_CoordsCalc = true;
	}
	Num--;
	Num *= 4;
	*x1 = m_coords[Num++];
	*y1 = m_coords[Num++];
	*x2 = m_coords[Num++];
	*y2 = m_coords[Num++];
	return true;
}

gcpAtom* gcpBond::GetAtomAt(double x, double y)
{
	double x1, y1;
	m_Begin->GetCoords(&x1, &y1);
	if ((fabs(x - x1) < 10) && (fabs(y - y1) < 10)) return (gcpAtom*)m_Begin;
	m_End->GetCoords(&x1, &y1);
	if ((fabs(x - x1) < 10) && (fabs(y - y1) < 10)) return (gcpAtom*)m_End;
	return NULL;
}

bool gcpBond::SaveNode(xmlDocPtr xml, xmlNodePtr node)
{
	switch(m_type)
	{
		case UpBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"up"); break;
		case DownBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"down"); break;
		case UndeterminedBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"undetermined"); break;
	}
	return true;
}

bool gcpBond::LoadNode(xmlNodePtr node)
{
	char* tmp;
	tmp = (char*)xmlGetProp(node, (xmlChar*)"type");
	if (!tmp) SetType(NormalBondType);
	else if (!strcmp(tmp, "up")) SetType(UpBondType);
	else if (!strcmp(tmp, "down")) SetType(DownBondType);
	else if (!strcmp(tmp, "undetermined")) SetType(UndeterminedBondType);
	else SetType(NormalBondType);
	return true;
}

void gcpBond::IncOrder(int n)
{
	Bond::IncOrder(n);
	if (m_order == 4) m_order = 1;//avoid quadruple bonds for now
	m_CoordsCalc = false;
	((gcpAtom*)m_Begin)->Update();
	((gcpAtom*)m_End)->Update();
}

double gcpBond::GetDist(double x, double y)
{
	double x1, y1, x2, y2, l, d1, d2;
	m_Begin->GetCoords(&x1, &y1);
	m_End->GetCoords(&x2, &y2);
	d1 = (x2 - x1) * (x1 - x) + (y2 - y1) * (y1 - y);
	d2 = (x2 - x1) * (x2 - x) + (y2 - y1) * (y2 - y);
	if ((d1 < 0.0) && (d2 < 0.0)) return sqrt(square(x2 - x) + square(y2 - y));
	if ((d1 > 0.0) && (d2 > 0.0)) return sqrt(square(x1 - x) + square(y1 - y));
	x2 -= x1;
	y2 -= y1;
	x -= x1;
	y -= y1;
	l = fabs(x2 * y - y2 * x) / sqrt(square(x2) + square(y2));
	return (l < DefaultBondDist * (m_order - 1)) ? 0 : l - DefaultBondDist * (m_order - 1);
}

void gcpBond::AddCycle(gcpCycle* pCycle)
{
	m_Cycles.push_back(pCycle);
	if ((m_order == 2) && m_CoordsCalc) SetDirty();
}

void gcpBond::RemoveCycle(gcpCycle* pCycle)
{
	m_Cycles.remove(pCycle);
	if ((m_order == 2) && m_CoordsCalc) SetDirty();
}

void gcpBond::SetDirty()
{
	gcpDocument *pDoc = (gcpDocument*)GetDocument();
	if (pDoc) pDoc->NotifyDirty(this);
	m_CoordsCalc = false;
}

void gcpBond::RemoveAllCycles()
{
	m_Cycles.clear();
	if (m_order == 2)
	{
		gcpDocument *pDoc = (gcpDocument*)GetDocument();
		if (pDoc) pDoc->NotifyDirty(this);
		m_CoordsCalc = false;
	}
}

void gcpBond::Move(double x, double y, double z)
{
	m_CoordsCalc = false;
}

void gcpBond::Revert()
{
	m_CoordsCalc = false;
	Atom* pAtom = m_Begin;
	m_Begin = m_End;
	m_End = pAtom;
}

double gcpBond::Get2DLength()
{
	double x1, y1, x2, y2;
	m_Begin->GetCoords(&x1, &y1);
	m_End->GetCoords(&x2, &y2);
	return sqrt(square(x1 - x2) + square(y1 - y2));
}
