// -*- C++ -*-

/* 
 * GChemPaint
 * erasertool.cc
 *
 * Copyright (C) 2001-2003
 *
 * Developed by Jean Brfort <jean.brefort@ac-dijon.fr>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "config.h"
#include <math.h>
#include "erasertool.h"
#include "settings.h"
#include "mendeleiev.h"
#include "document.h"
#include "atom.h"
#include "bond.h"

gcpEraserTool EraserTool;

gcpEraserTool::gcpEraserTool(): gcpTool(EraserId)
{
	m_bChanged = false;
}

gcpEraserTool::~gcpEraserTool()
{
}

void gcpEraserTool::OnClicked()
{
	if (m_pObject)
	{
		m_pItem = m_pView->GetCanvasItem(m_pWidget, m_pObject);
		GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(m_pItem), *cgroup;
		std::map<Atom*, Bond*>::iterator i;
		if (group)
		{
			GList* il = group->item_list, *cil;
			TypeId Id = m_pObject->GetType();
			gcpBond* pBond;
			switch (Id)
			{
				case BondType:
					while (il)
					{
						g_object_set(G_OBJECT(il->data), "fill_color", DeleteColor, NULL);
						il = il->next;
					}
					break;
				case AtomType:
					while (il)
					{
						if (GNOME_IS_CANVAS_GROUP(il->data))
						{
							cgroup = (GnomeCanvasGroup*)il->data;
							cil = cgroup->item_list;
							while (cil)
							{
								if (!GNOME_IS_CANVAS_RE(cil->data))
									g_object_set(G_OBJECT(cil->data),
										"fill_color", DeleteColor, NULL);
								else if (GNOME_IS_CANVAS_ELLIPSE(cil->data))
									g_object_set(G_OBJECT(cil->data),
										"outline_color", DeleteColor, NULL);
								cil = cil->next;

							}
						}
						else if (!GNOME_IS_CANVAS_RE(il->data)) g_object_set(G_OBJECT(il->data), "fill_color", DeleteColor, NULL);
						il = il->next;
					}
					pBond = (gcpBond*)((gcpAtom*)m_pObject)->GetFirstBond(i);
					while (pBond)
					{
						group = GNOME_CANVAS_GROUP(m_pView->GetCanvasItem(m_pWidget, pBond));
						il = group->item_list;
						while (il)
						{
							g_object_set(G_OBJECT(il->data), "fill_color", DeleteColor, NULL);
							il = il->next;
						}
						pBond = (gcpBond*)((gcpAtom*)m_pObject)->GetNextBond(i);
					}
					break;
				default:
					m_pObject->SetSelected(m_pWidget, SelStateErasing);
			}
			m_bChanged = true;
		}
	}
}

void gcpEraserTool::OnDrag()
{
	if (!m_pObject) return;
	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(m_pItem);
	GList* il;
	il = group->item_list;
	TypeId Id = m_pObject->GetType();
	GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
	Object* pObject = NULL;
	switch (Id)
	{
		case BondType:
			if (((gcpBond*)m_pObject)->GetDist(m_x / m_dZoomFactor, m_y / m_dZoomFactor) < (DefaultPadding + DefaultBondWidth / 2) / m_dZoomFactor)
			{
				if (!m_bChanged)
				{
					while (il)
					{
						g_object_set(G_OBJECT(il->data), "fill_color", DeleteColor, NULL);
						il = il->next;
					}
					m_bChanged = true;
				}
			}
			else if (m_bChanged)
			{
					while (il)
					{
						g_object_set(G_OBJECT(il->data), "fill_color", (gchar*)"black", NULL);
						il = il->next;
					}
				m_bChanged = false;
			}
			break;
		case AtomType:
//			GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
			if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
//			Object* pObject = NULL;
			if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
			if (pObject)
			{
				if (pObject->GetType() == BondType)
				{
					pObject = ((gcpBond*)pObject)->GetAtomAt(m_x / m_dZoomFactor, m_y / m_dZoomFactor);
				}
			}
			if (pObject == m_pObject)
			{
				if (!m_bChanged)
				{
					while (il)
					{
						if (!GNOME_IS_CANVAS_RE(il->data)) g_object_set(G_OBJECT(il->data), "fill_color", DeleteColor, NULL);
						il = il->next;
					}
					std::map<Atom*, Bond*>::iterator i;
					gcpBond* pBond = (gcpBond*)((gcpAtom*)m_pObject)->GetFirstBond(i);
					while (pBond)
					{
						group = GNOME_CANVAS_GROUP(m_pView->GetCanvasItem(m_pWidget, pBond));
						il = group->item_list;
						while (il)
						{
							g_object_set(G_OBJECT(il->data), "fill_color", DeleteColor, NULL);
							il = il->next;
						}
						pBond = (gcpBond*)((gcpAtom*)m_pObject)->GetNextBond(i);
					}
					m_bChanged = true;
				}
			}
			else if (m_bChanged)
			{
				while (il)
				{
					if (!GNOME_IS_CANVAS_RE(il->data)) g_object_set(G_OBJECT(il->data), "fill_color", (gchar*)"black", NULL);
					il = il->next;
				}
				std::map<Atom*, Bond*>::iterator i;
				gcpBond* pBond = (gcpBond*)((gcpAtom*)m_pObject)->GetFirstBond(i);
				while (pBond)
				{
					group = GNOME_CANVAS_GROUP(m_pView->GetCanvasItem(m_pWidget, pBond));
					il = group->item_list;
					while (il)
					{
						g_object_set(G_OBJECT(il->data), "fill_color", (gchar*)"black", NULL);
						il = il->next;
					}
					pBond = (gcpBond*)((gcpAtom*)m_pObject)->GetNextBond(i);
				}
				m_bChanged = false;
			}
			break;
		default:
			if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
			if (pObject == m_pObject)
			{
				if (!m_bChanged)
				{
					m_pObject->SetSelected(m_pWidget, SelStateErasing);
					m_bChanged = true;
				}
			}
			else if (m_bChanged)
			{
				m_pObject->SetSelected(m_pWidget, SelStateUnselected);
				m_bChanged = false;
			}
			
	}
}

void gcpEraserTool::OnRelease()
{
	if ((!m_pObject) || (!m_bChanged))
	{
		m_pItem = NULL;
		return;
	}
	gcpDocument* pDoc = m_pView->GetDoc();
	pDoc->Remove(m_pObject);
	m_pItem = NULL;
}
