// -*- C++ -*-

/* 
 * GChemPaint
 * texttool.cc 
 *
 * Copyright (C) 2002-2003
 *
 * Developed by Jean Brfort <jean.brefort@ac-dijon.fr>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "config.h"
#include "texttool.h"
#include "text.h"
#include "document.h"
#include "globals.h"

static Object* CreateText()
{
	return new gcpText();
}

gcpTextTool TextTool;
GtkTextTagTable *TextTagTable = NULL;

static bool on_toggled(GtkToggleButton *btn, gpointer* data)
{
	return TextTool.OnToggled(btn);
}

gcpTextTool::gcpTextTool(gcpToolId Id): gcpTool(Id)
{
	m_Active = NULL;
	g_type_init();
	//Text tags creation
	if (TextTagTable)
		g_object_ref(TextTagTable);
	else
	{
	TextTagTable = gtk_text_tag_table_new();
	GtkTextTag* tag = gtk_text_tag_new("bold");
	g_object_set(G_OBJECT(tag), "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_tag_table_add(TextTagTable, tag);
	g_object_unref((GObject*)tag);
	tag = gtk_text_tag_new("italic");
	g_object_set(G_OBJECT(tag), "style", PANGO_STYLE_ITALIC, NULL);
	gtk_text_tag_table_add(TextTagTable, tag);
	g_object_unref((GObject*)tag);
	tag = gtk_text_tag_new("underline");
	g_object_set(G_OBJECT(tag), "underline", PANGO_UNDERLINE_SINGLE, NULL);
	gtk_text_tag_table_add(TextTagTable, tag);
	g_object_unref((GObject*)tag);
	tag = gtk_text_tag_new("strikethrough");
	g_object_set(G_OBJECT(tag), "strikethrough", true, NULL);
	gtk_text_tag_table_add(TextTagTable, tag);
	g_object_unref((GObject*)tag);
	tag = gtk_text_tag_new("subscript");
	g_object_set(G_OBJECT(tag), "rise", -2 * PANGO_SCALE, "size", 8 * PANGO_SCALE, NULL);
	gtk_text_tag_table_add(TextTagTable, tag);
	g_object_unref((GObject*)tag);
	tag = gtk_text_tag_new("superscript");
	g_object_set(G_OBJECT(tag), "rise", 4 * PANGO_SCALE, "size", 8 * PANGO_SCALE, NULL);
	gtk_text_tag_table_add(TextTagTable, tag);
	g_object_unref((GObject*)tag);
	}
	if (Id ==TextId) Object::AddType("text", CreateText, TextType);
}

gcpTextTool::~gcpTextTool()
{
	g_object_unref(G_OBJECT(TextTagTable));
}

void gcpTextTool::OnClicked()
{
	void *func;
	if (m_Active && ((m_pObject == NULL) || (m_pObject->GetType() != TextType) || (m_Active != m_pData->Items[m_pObject]->item_list->data)))
	{
		Unselect();
	}
	if (!m_pObject)
	{
		gcpText *text = new gcpText(m_x0 / m_pData->ZoomFactor, m_y0 / m_pData->ZoomFactor);
		m_pView->GetDoc()->AddObject(text);
		m_pObject = text;
	}
	if (m_pObject)
	{
		if (m_pObject->GetType() != TextType) return;
		m_pObject->SetSelected(m_pWidget, SelStateUpdating);
		m_Active = GNOME_CANVAS_RICH_TEXT(GNOME_CANVAS_RICH_TEXT(g_object_get_data(G_OBJECT(m_pData->Items[m_pObject]), "text")));
		m_pView->SetGnomeCanvasRichTextActive(m_Active);
		g_object_set(G_OBJECT(m_Active), "editable", true, "cursor_visible", true, NULL);
	}
}

void gcpTextTool::OnDrag()
{
}

void gcpTextTool::OnRelease()
{
	
}

bool gcpTextTool::OnEvent(GdkEvent* event)
{
	if (m_Active)
	{
		if ((event->type == GDK_KEY_PRESS) || (event->type == GDK_KEY_RELEASE))
		{
			if (event->key.state & GDK_CONTROL_MASK)
			{
				switch(event->key.keyval)
				{
					case GDK_Right:
					case GDK_Left:
					case GDK_Up:
					case GDK_Down:
					case GDK_End:
					case GDK_Home:
					case GDK_Delete:
					case GDK_KP_Delete:
					case GDK_BackSpace:
					break;
					default:
						return false;
				}
			}
			if (event->key.keyval == GDK_KP_Enter || event->key.keyval == GDK_Return) return true;
			if (!g_utf8_validate(((GdkEventKey*)event)->string, -1, NULL))
			{
				gsize r, w;
				gchar* newstr = g_locale_to_utf8(((GdkEventKey*)event)->string, ((GdkEventKey*)event)->length, &r, &w, NULL);
				g_free(((GdkEventKey*)event)->string);
				((GdkEventKey*)event)->string = newstr;
				((GdkEventKey*)event)->length = w;
			}
			gnome_canvas_item_grab_focus((GnomeCanvasItem*)m_Active);
			GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS(((GTypeInstance*)m_Active)->g_class);
			GtkTextBuffer* buf = (GtkTextBuffer*)((gcpText*)m_pObject)->GetTextBuffer();
			gtk_text_buffer_begin_user_action(buf);
			klass->event((GnomeCanvasItem*)m_Active, event);
			gtk_text_buffer_end_user_action(buf);
			return true;
		}
	}
	return false;
}

void gcpTextTool::Activate()
{
	g_signal_connect(G_OBJECT(BoldBtn),"toggled", G_CALLBACK(on_toggled), NULL);
	g_signal_connect(G_OBJECT(ItalicBtn),"toggled", G_CALLBACK(on_toggled), NULL);
	g_signal_connect(G_OBJECT(UnderlineBtn),"toggled", G_CALLBACK(on_toggled), NULL);
	g_signal_connect(G_OBJECT(StrikethroughBtn),"toggled", G_CALLBACK(on_toggled), NULL);
	g_signal_connect(G_OBJECT(SubscriptBtn),"toggled", G_CALLBACK(on_toggled), NULL);
	g_signal_connect(G_OBJECT(SuperscriptBtn),"toggled", G_CALLBACK(on_toggled), NULL);
	gtk_widget_set_sensitive(TextToolBar, true);
}

void gcpTextTool::Deactivate()
{
	if (m_Active) Unselect();
	gtk_widget_set_sensitive(TextToolBar, false);
}

void gcpTextTool::NotifyViewChange()
{
	if (m_Active)
	{
		Unselect();
		sleep(1);
		while(gtk_events_pending()) gtk_main_iteration();
	}
}

void gcpTextTool::Unselect()
{
	if (!m_Active) return;
	g_object_set(G_OBJECT(m_Active), "editable", false, "cursor_visible", false, NULL);
	m_pView->SetGnomeCanvasRichTextActive(NULL);
	Object *pObj = (Object*)g_object_get_data(G_OBJECT(m_Active), "object");
	pObj->SetSelected(m_pWidget, SelStateUnselected);
	GtkTextBuffer* buf = gnome_canvas_rich_text_get_buffer(m_Active);
	m_Active = NULL;
	GtkTextIter start, end;
	gtk_text_buffer_get_start_iter(buf, &start);
	gtk_text_buffer_place_cursor(buf, &start);
	gtk_text_buffer_get_end_iter(buf, &end);
	char* text = gtk_text_buffer_get_text(buf, &start, &end, true);
	while(gtk_events_pending()) gtk_main_iteration();
	if (!*text) m_pView->GetDoc()->Remove(pObj);
}

bool gcpTextTool::OnToggled(GtkToggleButton *btn)
{
	if (!m_Active) return true;
	GtkTextBuffer* buf = gnome_canvas_rich_text_get_buffer(m_Active);
	GtkTextIter start, end;
	gtk_text_buffer_get_selection_bounds(buf, &start, &end);
	char* name;
	if (btn == (GtkToggleButton*)BoldBtn) name = "bold";
	else if (btn == (GtkToggleButton*)ItalicBtn) name = "italic";
	else if (btn == (GtkToggleButton*)UnderlineBtn) name = "underline";
	else if (btn == (GtkToggleButton*)StrikethroughBtn) name = "strikethrough";
	else if (btn == (GtkToggleButton*)SubscriptBtn) name = "subscript";
	else if (btn == (GtkToggleButton*)SuperscriptBtn) name = "superscript";
	else return false;
	if (gtk_toggle_button_get_active(btn))
		gtk_text_buffer_apply_tag_by_name(buf, name, &start, &end);
	else
		gtk_text_buffer_remove_tag_by_name(buf, name, &start, &end);
		
		
	return true;
}
